@@ -23,10 +23,14 @@ from botocore.vendored.requests import models
 from botocore.vendored.requests.sessions import REDIRECT_STATI
 from botocore.compat import HTTPHeaders, HTTPResponse
 from botocore.exceptions import UnseekableStreamError
-from botocore.vendored.requests.packages.urllib3.connection import VerifiedHTTPSConnection
-from botocore.vendored.requests.packages.urllib3.connection import HTTPConnection
-from botocore.vendored.requests.packages.urllib3.connectionpool import HTTPConnectionPool
-from botocore.vendored.requests.packages.urllib3.connectionpool import HTTPSConnectionPool
+from botocore.vendored.requests.packages.urllib3.connection import \
+    VerifiedHTTPSConnection
+from botocore.vendored.requests.packages.urllib3.connection import \
+    HTTPConnection
+from botocore.vendored.requests.packages.urllib3.connectionpool import \
+    HTTPConnectionPool
+from botocore.vendored.requests.packages.urllib3.connectionpool import \
+    HTTPSConnectionPool
 
 
 logger = logging.getLogger(__name__)
@@ -90,8 +94,8 @@ class AWSHTTPConnection(HTTPConnection):
         for header, value in self._tunnel_headers.iteritems():
             self.send("%s: %s\r\n" % (header, value))
         self.send("\r\n")
-        response = self.response_class(self.sock, strict = self.strict,
-                                       method = self._method)
+        response = self.response_class(self.sock, strict=self.strict,
+                                       method=self._method)
         (version, code, message) = response._read_status()
 
         if code != 200:
@@ -107,7 +111,7 @@ class AWSHTTPConnection(HTTPConnection):
 
     def _send_request(self, method, url, body, headers):
         self._response_received = False
-        if headers.get('Expect', '') == '100-continue':
+        if headers.get('Expect', b'') == b'100-continue':
             self._expect_header_set = True
         else:
             self._expect_header_set = False
@@ -117,9 +121,22 @@ class AWSHTTPConnection(HTTPConnection):
         self._expect_header_set = False
         return rval
 
+    def _convert_to_bytes(self, mixed_buffer):
+        # Take a list of mixed str/bytes and convert it
+        # all into a single bytestring.
+        # Any six.text_types will be encoded as utf-8.
+        bytes_buffer = []
+        for chunk in mixed_buffer:
+            if isinstance(chunk, six.text_type):
+                bytes_buffer.append(chunk.encode('utf-8'))
+            else:
+                bytes_buffer.append(chunk)
+        msg = b"\r\n".join(bytes_buffer)
+        return msg
+
     def _send_output(self, message_body=None):
         self._buffer.extend((b"", b""))
-        msg = b"\r\n".join(self._buffer)
+        msg = self._convert_to_bytes(self._buffer)
         del self._buffer[:]
         # If msg and message_body are sent in a single send() call,
         # it will avoid performance problems caused by the interaction
@@ -154,6 +171,18 @@ class AWSHTTPConnection(HTTPConnection):
             # we must run the risk of Nagle.
             self.send(message_body)
 
+    def _consume_headers(self, fp):
+        # Most servers (including S3) will just return
+        # the CLRF after the 100 continue response.  However,
+        # some servers (I've specifically seen this for squid when
+        # used as a straight HTTP proxy) will also inject a
+        # Connection: keep-alive header.  To account for this
+        # we'll read until we read '\r\n', and ignore any headers
+        # that come immediately after the 100 continue response.
+        current = None
+        while current != b'\r\n':
+            current = fp.readline()
+
     def _handle_expect_response(self, message_body):
         # This is called when we sent the request headers containing
         # an Expect: 100-continue header and received a response.
@@ -163,9 +192,9 @@ class AWSHTTPConnection(HTTPConnection):
             maybe_status_line = fp.readline()
             parts = maybe_status_line.split(None, 2)
             if self._is_100_continue_status(maybe_status_line):
-                # Read an empty line as per the RFC.
-                fp.readline()
-                logger.debug("100 Continue response seen, now sending request body.")
+                self._consume_headers(fp)
+                logger.debug("100 Continue response seen, "
+                             "now sending request body.")
                 self._send_message_body(message_body)
             elif len(parts) == 3 and parts[0].startswith(b'HTTP/'):
                 # From the RFC:
@@ -181,7 +210,7 @@ class AWSHTTPConnection(HTTPConnection):
                 # whatever the server has sent back is the final response
                 # and don't send the message_body.
                 logger.debug("Received a non 100 Continue response "
-                            "from the server, NOT sending request body.")
+                             "from the server, NOT sending request body.")
                 status_tuple = (parts[0].decode('ascii'),
                                 int(parts[1]), parts[2].decode('ascii'))
                 response_class = functools.partial(
@@ -206,8 +235,8 @@ class AWSHTTPConnection(HTTPConnection):
         parts = maybe_status_line.split(None, 2)
         # Check for HTTP/<version> 100 Continue\r\n
         return (
-            len(parts) == 3 and parts[0].startswith(b'HTTP/') and
-            parts[1] == b'100' and parts[2].startswith(b'Continue'))
+            len(parts) >= 3 and parts[0].startswith(b'HTTP/') and
+            parts[1] == b'100')
 
 
 class AWSHTTPSConnection(VerifiedHTTPSConnection):
@@ -234,6 +263,14 @@ class AWSRequest(models.RequestEncodingMixin, models.Request):
             for key, value in self.headers.items():
                 headers[key] = value
         self.headers = headers
+        # This is a dictionary to hold information that is used when
+        # processing the request. What is inside of ``context`` is open-ended.
+        # For example, it may have a timestamp key that is used for holding
+        # what the timestamp is when signing the request. Note that none
+        # of the information that is inside of ``context`` is directly
+        # sent over the wire; the information is only used to assist in
+        # creating what is sent over the wire.
+        self.context = {}
 
     def prepare(self):
         """Constructs a :class:`AWSPreparedRequest <AWSPreparedRequest>`."""
