@@ -19,10 +19,23 @@ from botocore.compat import zip
 from botocore.utils import set_value_from_jmespath, merge_dicts
 
 
+class PaginatorModel(object):
+    def __init__(self, paginator_config):
+        self._paginator_config = paginator_config['pagination']
+
+    def get_paginator(self, operation_name):
+        try:
+            single_paginator_config = self._paginator_config[operation_name]
+        except KeyError:
+            raise ValueError("Paginator for operation does not exist: %s"
+                             % operation_name)
+        return single_paginator_config
+
+
 class PageIterator(object):
     def __init__(self, method, input_token, output_token, more_results,
-                 result_keys, non_aggregate_keys, max_items, starting_token,
-                 page_size, op_kwargs):
+                 result_keys, non_aggregate_keys, limit_key, max_items,
+                 starting_token, page_size, op_kwargs):
         self._method = method
         self._op_kwargs = op_kwargs
         self._input_token = input_token
@@ -30,6 +43,7 @@ class PageIterator(object):
         self._more_results = more_results
         self._result_keys = result_keys
         self._max_items = max_items
+        self._limit_key = limit_key
         self._starting_token = starting_token
         self._page_size = page_size
         self._op_kwargs = op_kwargs
@@ -135,11 +149,9 @@ class PageIterator(object):
             next_token = self._parse_starting_token()[0]
             self._inject_token_into_kwargs(op_kwargs, next_token)
         if self._page_size is not None:
-            # Determine the proper parameter name for limiting
-            # page size.
-            page_size_name = self._operation.pagination['limit_key']
-            # Pass the page size as the determined parameter name.
-            op_kwargs[page_size_name] = self._page_size
+            # Pass the page size as the parameter name for limiting
+            # page size, also known as the limit_key.
+            op_kwargs[self._limit_key] = self._page_size
 
     def _inject_token_into_kwargs(self, op_kwargs, next_token):
         for name, token in zip(self._input_token, next_token):
@@ -213,7 +225,15 @@ class PageIterator(object):
         for result_expression in self.result_keys:
             set_value_from_jmespath(complete_result,
                                     result_expression.expression, [])
-        for _, page in self:
+        for response in self:
+            page = response
+            # We want to try to catch operation object pagination
+            # and format correctly for those. They come in the form
+            # of a tuple of two elements: (http_response, parsed_responsed).
+            # We want the parsed_response as that is what the page iterator
+            # uses. We can remove it though once operation objects are removed.
+            if isinstance(response, tuple) and len(response) == 2:
+                page = response[1]
             # We're incrementally building the full response page
             # by page.  For each page in the response we need to
             # inject the necessary components from the page
@@ -268,6 +288,7 @@ class Paginator(object):
         self._non_aggregate_keys = self._get_non_aggregate_keys(
             self._pagination_cfg)
         self._result_keys = self._get_result_keys(self._pagination_cfg)
+        self._limit_key = self._get_limit_key(self._pagination_cfg)
 
     @property
     def result_keys(self):
@@ -307,6 +328,9 @@ class Paginator(object):
             result_key = [jmespath.compile(rk) for rk in result_key]
             return result_key
 
+    def _get_limit_key(self, config):
+        return config.get('limit_key')
+
     def paginate(self, **kwargs):
         """Create paginator object for an operation.
 
@@ -320,6 +344,7 @@ class Paginator(object):
             self._method, self._input_token,
             self._output_token, self._more_results,
             self._result_keys, self._non_aggregate_keys,
+            self._limit_key,
             page_params['max_items'],
             page_params['starting_token'],
             page_params['page_size'],
@@ -373,12 +398,12 @@ class ResultKeyIterator(object):
 class DeprecatedPageIterator(PageIterator):
     def __init__(self, operation, endpoint, input_token,
                  output_token, more_results,
-                 result_keys, non_aggregate_keys, max_items, starting_token,
-                 page_size, op_kwargs):
+                 result_keys, non_aggregate_keys, limit_key, max_items,
+                 starting_token, page_size, op_kwargs):
         super(DeprecatedPageIterator, self).__init__(
             None, input_token, output_token, more_results, result_keys,
-            non_aggregate_keys, max_items, starting_token, page_size,
-            op_kwargs)
+            non_aggregate_keys, limit_key, max_items,
+            starting_token, page_size, op_kwargs)
         self._operation = operation
         self._endpoint = endpoint
 
@@ -410,6 +435,7 @@ class DeprecatedPaginator(Paginator):
             self._operation, endpoint, self._input_token,
             self._output_token, self._more_results,
             self._result_keys, self._non_aggregate_keys,
+            self._limit_key,
             page_params['max_items'],
             page_params['starting_token'],
             page_params['page_size'],
