@@ -43,7 +43,7 @@ import base64
 from xml.etree import ElementTree
 import calendar
 
-import six
+from botocore.compat import six
 
 from botocore.compat import json, formatdate
 from botocore.utils import parse_timestamp, parse_to_aware_datetime
@@ -137,7 +137,7 @@ class Serializer(object):
         return int(calendar.timegm(value.timetuple()))
 
     def _timestamp_rfc822(self, value):
-        return formatdate(value)
+        return formatdate(value, usegmt=True)
 
     def _convert_timestamp_to_str(self, value):
         datetime_obj = parse_to_aware_datetime(value)
@@ -317,7 +317,7 @@ class JSONSerializer(Serializer):
             # of the passed in serialized dict.  We'll then add
             # all the structure members as key/vals in the new serialized
             # dictionary we just created.
-            new_serialized = {}
+            new_serialized = self.MAP_TYPE()
             serialized[key] = new_serialized
             serialized = new_serialized
         members = shape.members
@@ -331,6 +331,18 @@ class JSONSerializer(Serializer):
         for sub_key, sub_value in value.items():
             self._serialize(map_obj, sub_value, shape.value, sub_key)
 
+    def _serialize_type_list(self, serialized, value, shape, key):
+        list_obj = []
+        serialized[key] = list_obj
+        for list_item in value:
+            wrapper = {}
+            # The JSON list serialization is the only case where we aren't
+            # setting a key on a dict.  We handle this by using
+            # a __current__ key on a wrapper dict to serialize each
+            # list item before appending it to the serialized list.
+            self._serialize(wrapper, list_item, shape.member, "__current__")
+            list_obj.append(wrapper["__current__"])
+
     def _default_serialize(self, serialized, value, shape, key):
         serialized[key] = value
 
@@ -433,7 +445,7 @@ class BaseRestSerializer(Serializer):
             # If there's a payload member, we serialized that
             # member to they body.
             body_params = parameters.get(payload_member)
-            if body_params:
+            if body_params is not None:
                 serialized['body'] = self._serialize_body_params(
                     body_params,
                     shape_members[payload_member])
@@ -489,10 +501,12 @@ class BaseRestSerializer(Serializer):
             return value
 
 
-class RestJSONSerializer(BaseRestSerializer):
+class RestJSONSerializer(BaseRestSerializer, JSONSerializer):
 
     def _serialize_body_params(self, params, shape):
-        return json.dumps(params)
+        serialized_body = self.MAP_TYPE()
+        self._serialize(serialized_body, params, shape)
+        return json.dumps(serialized_body)
 
 
 class RestXMLSerializer(BaseRestSerializer):
